/* exported init */
/* exported enable */
/* exported disable */

"use strict";

const ExtensionUtils = imports.misc.extensionUtils;
const Extension = ExtensionUtils.getCurrentExtension();

const { Clutter, Gio, GLib, GObject, St } = imports.gi;
const { main, panelMenu } = imports.ui;


function getPingRTT(pingResult) {
	pingResult = pingResult.trim().split(/\n/);
	pingResult = pingResult.pop().toString();          // rtt min/avg/max/mdev = 13.109/21.812/38.923/9.078 ms
	pingResult = pingResult.split(/\s/)[3].toString(); // 13.109/21.812/38.923/9.078
	pingResult = pingResult.split(/\//);

	const rttMin = Math.round(pingResult[0]);
	const rttAvg = Math.round(pingResult[1]);
	const rrtMax = Math.round(pingResult[2]);

	return [rttMin, rttAvg, rrtMax];
}


const LatencyIndicator = GObject.registerClass(
	class LatencyIndicator extends panelMenu.Button {
		_init() {
			super._init(St.Align.START, "LatencyIndicator", false);
			this.settings = ExtensionUtils.getSettings(Extension.metadata["settings-schema"]);

			this.icon = new St.Icon({
				gicon: new Gio.ThemedIcon({ name: "network-transmit-receive" }),
				style_class: "system-status-icon"
			});

			this.label = new St.Label({
				y_align: Clutter.ActorAlign.CENTER,
				y_expand: true
			});
			this.label.text = "...";
			this._update();

			this.box = new St.BoxLayout();
			this.box.add(this.icon);
			this.box.add(this.label);

			this.add_child(this.box);
		}

		_update() {
			try {
				const argv = ["ping", "-c", "3", this.settings.get_string("ping-destination")];
				const flags = Gio.SubprocessFlags.STDOUT_PIPE | Gio.SubprocessFlags.STDERR_PIPE;
				const process = Gio.Subprocess.new(argv, flags);

				process.communicate_utf8_async(null, null, (process, result) => {
					try {
						const [, stdout, stderr] = process.communicate_utf8_finish(result);
						const exitStatus = process.get_exit_status();

						const EXIT_STATUS_TIMEOUT = 1;
						const EXIT_STATUS_ERROR = 2;

						switch (exitStatus) {
							case EXIT_STATUS_TIMEOUT:
								this.label.text = "Timeout";
								throw new Error(stdout);
							case EXIT_STATUS_ERROR:
								this.label.text = "Error";
								throw new Error(stderr);
						}

						const [, rttAvg] = getPingRTT(stdout);

						this.label.text = `${rttAvg} ms`;
					} catch (error) {
						logError(error);
					}
				});
			} catch (error) {
				logError(error);
			} finally {
				// TODO: Replace constant with setting
				const updateDelay = 3;
				this._timeout = GLib.timeout_add_seconds(GLib.PRIORITY_DEFAULT, updateDelay, this._update.bind(this));
			}

			return GLib.SOURCE_REMOVE;
		}

		_onDestroy() {
			if (this._timeout) GLib.source_remove(this._timeout);

			super._onDestroy();
		}
	}
);


let latencyIndicator;

function init() {
	// do nothing
}

function enable() {
	latencyIndicator = new LatencyIndicator();
	main.panel.addToStatusArea("latencyIndicator", latencyIndicator);
}

function disable() {
	latencyIndicator.destroy();
}
