/* exported init */
/* exported buildPrefsWidget */

"use strict";

const ExtensionUtils = imports.misc.extensionUtils;
const Extension = ExtensionUtils.getCurrentExtension();

const { Gio, GObject, Gtk } = imports.gi;

const DEFAULT_MARGIN = 10;
const DEFAULT_SPACING = 5;


function buildPrefEntry(settings, settingLabel, settingKey) {
	const box = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL });
	const label = new Gtk.Label({ label: settingLabel, xalign: 0 });
	const entry = new Gtk.Entry({ text: settings.get_string(settingKey) });

	settings.bind(settingKey, entry.buffer, "text", Gio.SettingsBindFlags.DEFAULT);

	entry.set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, "document-revert");
	entry.connect("icon-press", () => {
		settings.reset(settingKey);
		entry.buffer.text = settings.get_string(settingKey);
	});

	box.pack_start(label, true, true, DEFAULT_SPACING);
	box.pack_start(entry, false, false, DEFAULT_SPACING);

	return box;
}


const LatencyMonitorPrefsWidget = GObject.registerClass(
	class LatencyMonitorPrefsWidget extends Gtk.Notebook {
		_init() {
			super._init();
			this.settings = ExtensionUtils.getSettings(Extension.metadata["settings-schema"]);

			this.append_page(this._buildLatencyPage(), Gtk.Label.new("Latency"));
		}

		_buildLatencyPage() {
			const page = new Gtk.Box({
				margin: DEFAULT_MARGIN,
				margin_top: DEFAULT_MARGIN,
				orientation: Gtk.Orientation.VERTICAL
			});

			const pingDestinationPref = buildPrefEntry(this.settings, "Ping Destination", "ping-destination");
			page.pack_start(pingDestinationPref, false, false, DEFAULT_SPACING);

			return page;
		}
	}
);


function init() {
	// do nothing
}

function buildPrefsWidget() {
	const prefsWidget = new LatencyMonitorPrefsWidget();
	prefsWidget.show_all();
	return prefsWidget;
}
